unit DprExpUnit;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  Menus, ComCtrls, Registry, CommCtrl;

Const
  NT_Group   = 0;
  NT_Project = 1;
  RegKey     = 'Software\Open Project Expert';

type
  TExpertForm = class(TForm)
    ProjectTree   : TTreeView;
    StatusBar     : TStatusBar;
    PopupMenu     : TPopupMenu;
    pmOpen        : TMenuItem;
    pmNewGroup    : TMenuItem;
    pmNewProject  : TMenuItem;
    pmDelete      : TMenuItem;
    N1            : TMenuItem;
    N2            : TMenuItem;
    pmExpandAll: TMenuItem;
    pmCollapseAll : TMenuItem;
    ImageList     : TImageList;
    OpenDialog: TOpenDialog;
    procedure FormCreate(Sender: TObject);
    procedure ProjectTreeEdited(Sender: TObject; Node: TTreeNode; var S: String);
    procedure pmOpenClick(Sender: TObject);
    procedure pmNewGroupClick(Sender: TObject);
    procedure pmNewProjectClick(Sender: TObject);
    procedure pmExpandAllClick(Sender: TObject);
    procedure pmCollapseAllClick(Sender: TObject);
    procedure pmDeleteClick(Sender: TObject);
    procedure ProjectTreeClick(Sender: TObject);
    procedure ProjectTreeEditing(Sender: TObject; Node: TTreeNode; var AllowEdit: Boolean);
    procedure ProjectTreeGetImageIndex(Sender: TObject; Node: TTreeNode);
    procedure PopupMenuPopup(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure ProjectTreeKeyUp(Sender: TObject; var Key: Word;
      Shift: TShiftState);
  private
    procedure SaveData;
    procedure LoadData;
    procedure EnabledMenu;
    procedure OpenProject(Name : string);
  public
  end;

function DPRExpert : TExpertForm;

implementation
uses ExptIntf;
{$R *.DFM}

var
  ExpertForm : TExpertForm = nil;

//--------------------------------------------------------------------
// Name     : DPRExpert
// Params   :
// Return   :
// Descript :
//--------------------------------------------------------------------
function DPRExpert : TExpertForm;
begin
  if not Assigned(ExpertForm) then ExpertForm := TExpertForm.Create(nil);
  Result := ExpertForm;
end;

//--------------------------------------------------------------------
// Name     : TExpertForm.FormCreate
// Params   :
// Return   :
// Descript :
//--------------------------------------------------------------------
procedure TExpertForm.FormCreate(Sender: TObject);
begin
  EnabledMenu;
  LoadData;
end;

//--------------------------------------------------------------------
// Name     : TExpertForm.FormClose
// Params   :
// Return   :
// Descript :
//--------------------------------------------------------------------
procedure TExpertForm.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  SaveData;
end;

//--------------------------------------------------------------------
// Name     : TExpertForm.ProjectTreeGetImageIndex
// Params   :
// Return   :
// Descript :
//--------------------------------------------------------------------
procedure TExpertForm.ProjectTreeGetImageIndex(Sender: TObject; Node: TTreeNode);
begin
  if Integer(Node.Data) = NT_Group then
  With Node do
  begin
    if Node.Expanded then
    begin
      ImageIndex    := 2;
      SelectedIndex := 2;
    end
    else
    begin
      ImageIndex    := 0;
      SelectedIndex := 0;
    end;
  end;

  if Integer(Node.Data) = NT_Project then
  With Node do
  begin
    ImageIndex    := 1;
    SelectedIndex := 1;
  end;
end;

//--------------------------------------------------------------------
// Name     : SaveData
// Params   :
// Return   :
// Descript :
//--------------------------------------------------------------------
procedure TExpertForm.SaveData;
var
  i   : integer;
  k   : integer;
  idx : integer;
  TR  : TRegIniFile;
  s   : string;
begin
  TR := TRegIniFile.Create('');
  try
    TR.RootKey := HKEY_CURRENT_USER;
    TR.OpenKey(RegKey, True);
    k := TR.ReadInteger('Projects', 'GroupCount', 0);
    for i := 0 to k - 1 do
      TR.EraseSection('Projects\Group' + IntToStr(i));

    //    
    idx := 0;
    s := 'Projects\';
    for i := 0 to ProjectTree.Items.Count - 1 do
    begin
      if ProjectTree.Items[i].Parent = nil then
      begin
        // Write Group
        TR.WriteString (s + 'Group' + IntToStr(idx), 'Text',  ProjectTree.Items[i].Text);
        TR.WriteInteger(s + 'Group' + IntToStr(idx), 'Count', ProjectTree.Items[i].Count);
        // Wriet Projects
        for k := 0 to ProjectTree.Items[i].Count - 1 do
          TR.WriteString (s + 'Group' + IntToStr(idx), 'Project' + IntToStr(k), ProjectTree.Items[i].Item[k].Text);
        Inc(idx);
      end;
    end;
    TR.WriteInteger('Projects','GroupCount', idx);
  finally
    TR.Free;
  end;
end;

//--------------------------------------------------------------------
// Name     : LoadData
// Params   :
// Return   :
// Descript :
//--------------------------------------------------------------------
procedure TExpertForm.LoadData;
var
  i      : integer;
  k      : integer;
  TN     : TTreeNode;
  s      : string;
  PCount : integer;
  GCount : integer;
begin
  With TRegIniFile.Create('') do
  try
    RootKey := HKEY_CURRENT_USER;
    OpenKey(RegKey, True);
    GCount := ReadInteger('Projects', 'GroupCount', 0);
    for i := 0 to GCount - 1 do
    begin
      s := 'Projects\Group';
      TN := ProjectTree.Items.AddObject(nil,
                                      ReadString(s + IntToStr(i), 'Text', ''),
                                      Pointer(NT_Group));
      PCount := ReadInteger(s + IntToStr(i), 'Count', 0);
      for k := 0 to PCount - 1 do
        ProjectTree.Items.AddChildObject(TN,
                                         ReadString(s + IntToStr(i), 'Project' + IntToStr(k), ''),
                                         Pointer(NT_Project));
    end;
  finally
    Free;
  end;
end;

//--------------------------------------------------------------------
// Name     : TExpertForm.EnabledMenu
// Params   :
// Return   :
// Descript :
//--------------------------------------------------------------------
procedure TExpertForm.EnabledMenu;
var
  Res : boolean;
begin
  Res := Assigned(ProjectTree.Selected);
  pmOpen.Enabled        := Res and (not ProjectTree.Selected.HasChildren);
  pmNewGroup.Enabled    := True;
  pmNewProject.Enabled  := Res;
  pmDelete.Enabled      := Res;
  pmExpandAll.Enabled   := ProjectTree.Items.Count > 0;
  pmCollapseAll.Enabled := ProjectTree.Items.Count > 0;
end;

//--------------------------------------------------------------------
// Name     : TExpertForm.OpenProject
// Params   :
// Return   :
// Descript :
//--------------------------------------------------------------------
procedure TExpertForm.OpenProject(Name : string);
begin
  if FileExists(Name) then
  begin
    if Assigned(ToolServices) then ToolServices.OpenProject(Name);
  end
  else MessageBox(Handle, PChar(Format('"%s" - not found', [Name])), PChar('Error'), MB_OK or MB_ICONSTOP);
end;

//--------------------------------------------------------------------
// Name     : TExpertForm.ProjectTreeEditing
// Params   :
// Return   :
// Descript :
//--------------------------------------------------------------------
procedure TExpertForm.ProjectTreeEditing(Sender: TObject; Node: TTreeNode; var AllowEdit: Boolean);
begin
  AllowEdit := False;
  if Assigned(ProjectTree.Selected)
  then AllowEdit := Integer(ProjectTree.Selected.Data) = NT_Group;
end;

//--------------------------------------------------------------------
// Name     : TExpertForm.ProjectTreeEdited
// Params   :
// Return   :
// Descript :
//--------------------------------------------------------------------
procedure TExpertForm.ProjectTreeEdited(Sender: TObject; Node: TTreeNode; var S: String);
begin
  Node.Text := S;
  SaveData;
end;

//--------------------------------------------------------------------
// Name     : TExpertForm.pmOpenClick
// Params   :
// Return   :
// Descript :
//--------------------------------------------------------------------
procedure TExpertForm.pmOpenClick(Sender: TObject);
var
  TN : TTreeNode;
begin
  TN := ProjectTree.Selected;
  if Assigned(TN) then
  begin
    if Integer(TN.Data) = NT_Project
    then OpenProject(TN.Text);
  end;
end;

//--------------------------------------------------------------------
// Name     : TExpertForm.pmNewGroupClick
// Params   :
// Return   :
// Descript :
//--------------------------------------------------------------------
procedure TExpertForm.pmNewGroupClick(Sender: TObject);
begin
  ProjectTree.Selected := ProjectTree.Items.AddObject(nil, 'New Group', Pointer(NT_Group));
  ProjectTree.Selected.EditText;
  SaveData;
end;

//--------------------------------------------------------------------
// Name     : TExpertForm.pmNewProjectClick
// Params   :
// Return   :
// Descript :
//--------------------------------------------------------------------
procedure TExpertForm.pmNewProjectClick(Sender: TObject);
var
  TN : TTreeNode;
  S  : string;
begin
  TN := ProjectTree.Selected;
  if OpenDialog.Execute then
  begin
    S := OpenDialog.FileName;
    if Assigned(TN) then
    begin
      case Integer(TN.Data) of
        NT_Group   : ProjectTree.Selected := ProjectTree.Items.AddChildObject(TN, S, Pointer(NT_Project));
        NT_Project : ProjectTree.Selected := ProjectTree.Items.AddObject(TN, S, Pointer(NT_Project));
      end;
      SaveData;
    end;
  end;
end;

//--------------------------------------------------------------------
// Name     : TExpertForm.pmExtractAllClick
// Params   :
// Return   :
// Descript :
//--------------------------------------------------------------------
procedure TExpertForm.pmExpandAllClick(Sender: TObject);
begin
  ProjectTree.FullExpand;
end;

//--------------------------------------------------------------------
// Name     : TExpertForm.pmCollapseAllClick
// Params   :
// Return   :
// Descript :
//--------------------------------------------------------------------
procedure TExpertForm.pmCollapseAllClick(Sender: TObject);
begin
  ProjectTree.FullCollapse;
end;

//--------------------------------------------------------------------
// Name     : TExpertForm.pmDeleteClick
// Params   :
// Return   :
// Descript :
//--------------------------------------------------------------------
procedure TExpertForm.pmDeleteClick(Sender: TObject);
begin
  if Assigned(ProjectTree.Selected) then
  begin
    if MessageBox(Handle,
                  PChar(Format('Delete "%s" ?', [ProjectTree.Selected.Text])),
                  PChar('Question'),
                  MB_YESNO or MB_ICONQUESTION) = IDYES then
    begin
      ProjectTree.Items.Delete(ProjectTree.Selected);
      SaveData;
    end;
  end;
end;

//--------------------------------------------------------------------
// Name     : TExpertForm.ProjectTreeClick
// Params   :
// Return   :
// Descript :
//--------------------------------------------------------------------
procedure TExpertForm.ProjectTreeClick(Sender: TObject);
begin
  EnabledMenu;
end;

//--------------------------------------------------------------------
// Name     : TExpertForm.PopupMenuPopup
// Params   :
// Return   :
// Descript :
//--------------------------------------------------------------------
procedure TExpertForm.PopupMenuPopup(Sender: TObject);
begin
  EnabledMenu;
end;

//--------------------------------------------------------------------
// Name     : TExpertForm.ProjectTreeKeyUp
// Params   : 
// Return   :
// Descript :
//--------------------------------------------------------------------
procedure TExpertForm.ProjectTreeKeyUp(Sender: TObject; var Key: Word; Shift: TShiftState);
begin
  if Key = VK_F2 then
  begin
    if Assigned(ProjectTree.Selected)
    then ProjectTree.Selected.EditText;
  end;
end;

//--------------------------------------------------------------------
initialization

finalization
  ExpertForm.Free;
  ExpertForm := nil;

end.
